from minet.cli.cookies import COOKIES_COMMAND
from minet.cli.crawl import CRAWL_COMMAND, FOCUS_CRAWL_COMMAND
from minet.cli.dump import DUMP_QUEUE_COMMAND, DUMP_URLS_COMMAND, DUMP_GRAPH_COMMAND
from minet.cli.extract import EXTRACT_COMMAND
from minet.cli.fetch import FETCH_COMMAND, RESOLVE_COMMAND
from minet.cli.scrape import SCRAPE_COMMAND
from minet.cli.url_extract import URL_EXTRACT_COMMAND
from minet.cli.url_join import URL_JOIN_COMMAND
from minet.cli.url_parse import URL_PARSE_COMMAND

from minet.cli.buzzsumo import BUZZSUMO_COMMAND
from minet.cli.crowdtangle import CROWDTANGLE_COMMAND
from minet.cli.facebook import FACEBOOK_COMMAND
from minet.cli.google import GOOGLE_COMMAND
from minet.cli.hyphe import HYPHE_COMMAND
from minet.cli.instagram import INSTAGRAM_COMMAND
from minet.cli.mediacloud import MEDIACLOUD_COMMAND
from minet.cli.telegram import TELEGRAM_COMMAND
from minet.cli.tiktok import TIKTOK_COMMAND
from minet.cli.twitter import TWITTER_COMMAND
from minet.cli.wikipedia import WIKIPEDIA_COMMAND
from minet.cli.youtube import YOUTUBE_COMMAND

MINET_COMMANDS = [
    COOKIES_COMMAND,
    CRAWL_COMMAND,
    FOCUS_CRAWL_COMMAND,
    DUMP_QUEUE_COMMAND,
    DUMP_URLS_COMMAND,
    DUMP_GRAPH_COMMAND,
    EXTRACT_COMMAND,
    FETCH_COMMAND,
    RESOLVE_COMMAND,
    SCRAPE_COMMAND,
    URL_EXTRACT_COMMAND,
    URL_JOIN_COMMAND,
    URL_PARSE_COMMAND,
    #
    BUZZSUMO_COMMAND,
    CROWDTANGLE_COMMAND,
    FACEBOOK_COMMAND,
    GOOGLE_COMMAND,
    HYPHE_COMMAND,
    INSTAGRAM_COMMAND,
    MEDIACLOUD_COMMAND,
    TELEGRAM_COMMAND,
    TIKTOK_COMMAND,
    TWITTER_COMMAND,
    WIKIPEDIA_COMMAND,
    YOUTUBE_COMMAND,
]
